@@ -80,7 +80,8 @@ struct _GeditDocumentClass
 					 gboolean can_redo);
 
 };
-
+#define GEDIT_ERROR_INVALID_UTF8_DATA 	1024
+#define GEDIT_ERROR_UNTITLED		1025	
 #define GEDIT_DOCUMENT_IO_ERROR gedit_document_io_error_quark ()
 GQuark 		gedit_document_io_error_quark (void);
 
@@ -92,6 +93,7 @@ GeditDocument* 	gedit_document_new_with_uri 	(const gchar *uri, GError **error);
 void 		gedit_document_set_readonly 	(GeditDocument *doc, gboolean readonly);
 gboolean	gedit_document_is_readonly 	(GeditDocument *doc);
 
+gchar*		gedit_document_get_raw_uri 	(const GeditDocument* doc);
 gchar*		gedit_document_get_uri 		(const GeditDocument* doc);
 gchar*		gedit_document_get_short_name 	(const GeditDocument* doc);
 
@@ -105,6 +107,9 @@ gboolean	gedit_document_save 		(GeditDocument* doc, GError **error);
 gboolean	gedit_document_save_as 		(GeditDocument* doc, 
 						 const gchar *uri, GError **error);
 
+gboolean	gedit_document_save_a_copy_as 	(GeditDocument* doc, 
+						 const gchar *uri, GError **error);
+
 gboolean 	gedit_document_revert 		(GeditDocument *doc,  GError **error);
 
 gboolean	gedit_document_is_untitled 	(const GeditDocument* doc);
@@ -125,6 +130,9 @@ void		gedit_document_insert_text_at_cursor (GeditDocument *doc,
                                                  const gchar   *text,
                                                  gint           len);
 
+gint		gedit_document_get_cursor (GeditDocument *doc);
+void		gedit_document_set_cursor (GeditDocument *doc, gint cursor);
+
 void 		gedit_document_delete_text 	(GeditDocument *doc, 
 		                                 gint start, gint end);
 
