@@ -46,10 +46,15 @@
 #include "gedit-prefs.h"
 #include "gedit-file-selector-util.h"
 #include "gedit-plugins-engine.h"
+#include "gnome-recent-model.h"
 
 static gboolean gedit_file_open_real (const gchar* file_name, GeditMDIChild* child);
 static gboolean gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child);
 
+
+static gchar* gedit_default_path = NULL;
+
+
 void 
 gedit_file_new (void)
 {
@@ -122,7 +127,7 @@ gedit_file_open (GeditMDIChild *active_child)
 			TRUE,
 		        _("Open File ..."), 
 			NULL, 
-			NULL);
+			gedit_default_path);
 	
 	if (files) 
 	{
@@ -130,7 +135,26 @@ gedit_file_open (GeditMDIChild *active_child)
 		for (i = 0; files[i]; ++i)
 		{
 			if (gedit_file_open_real (files[i], active_child))
-				gedit_utils_flash_va (_("Loaded file '%s'"), files[i]);
+			{
+				gchar *uri_utf8;
+
+				uri_utf8 = g_filename_to_utf8 (files[i], -1, NULL, NULL, NULL);
+
+				if (uri_utf8 != NULL)
+				{
+					gedit_utils_flash_va (_("Loaded file '%s'"), uri_utf8);
+
+					g_free (uri_utf8);
+				}
+
+				if (gedit_utils_uri_has_file_scheme (files[i]))
+				{				
+					if (gedit_default_path != NULL)
+						g_free (gedit_default_path);
+
+					gedit_default_path = gnome_vfs_x_uri_get_dirname (files[i]);
+				}				
+			}
 			
 			gedit_debug (DEBUG_FILE, "File: %s", files[i]);
 		}
@@ -143,14 +167,23 @@ static gboolean
 gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 {
 	GError *error = NULL;
-	gchar* uri;
-	GeditRecent *recent;
-	BonoboWindow *win;
+	gchar *uri;
+	gchar *uri_utf8;
+	GnomeRecentModel *recent;
 
 	gedit_debug (DEBUG_FILE, "File name: %s", file_name);
 
 	uri = gnome_vfs_x_make_uri_canonical (file_name);
 	g_return_val_if_fail (uri != NULL, FALSE);
+
+	uri_utf8 = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
+	if (uri_utf8 == NULL)
+	{
+		g_free (uri);
+
+		return FALSE;
+	}
+	
 	gedit_debug (DEBUG_FILE, "URI: %s", uri);
 
 	if (active_child == NULL ||
@@ -163,7 +196,7 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 
 		if (error)
 		{
-			gedit_utils_error_reporting_loading_file (uri, error,
+			gedit_utils_error_reporting_loading_file (uri_utf8, error,
 					GTK_WINDOW (gedit_get_active_window ()));
 			
 			g_error_free (error);
@@ -188,7 +221,7 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 
 		if (error)
 		{
-			gedit_utils_error_reporting_loading_file (uri, error,
+			gedit_utils_error_reporting_loading_file (uri_utf8, error,
 					GTK_WINDOW (gedit_get_active_window ()));
 			
 			g_error_free (error);
@@ -196,11 +229,11 @@ gedit_file_open_real (const gchar* file_name, GeditMDIChild* active_child)
 			return FALSE;
 		}	
 	}
+	
+	recent = gedit_recent_get_model ();
+	gnome_recent_model_add (recent, uri_utf8);
 
-	win = gedit_get_active_window ();
-	recent = gedit_mdi_get_recent_from_window (win);
-	gedit_recent_add (recent, uri);
-
+	g_free (uri_utf8);
 	g_free (uri);
 
 	gedit_debug (DEBUG_FILE, "END: %s\n", file_name);
@@ -269,19 +302,35 @@ gedit_file_save (GeditMDIChild* child)
 	}	
 	else
 	{
-		BonoboWindow *win;
-		GeditRecent *recent;
+		GnomeRecentModel *recent;
+		gchar *raw_uri;
+		gchar *canonical_uri;
+		gchar *canonical_uri_utf8;
 
 		gedit_debug (DEBUG_FILE, "OK");
 
 		gedit_utils_flash_va (_("File '%s' saved."), uri);
 
-		win = gedit_get_active_window ();
-		recent = gedit_mdi_get_recent_from_window (win);
-		gedit_recent_add (recent, uri);
-
 		g_free (uri);
 
+		raw_uri = gedit_document_get_raw_uri (doc);
+		g_return_val_if_fail (raw_uri != NULL, TRUE);
+		
+		canonical_uri = gnome_vfs_x_make_uri_canonical (raw_uri);
+		g_return_val_if_fail (canonical_uri != NULL, TRUE);
+
+		/* canonical_uri is not valid utf8 */
+		
+		canonical_uri_utf8 = g_filename_to_utf8 (canonical_uri, -1, NULL, NULL, NULL);
+		g_return_val_if_fail (canonical_uri_utf8 != NULL, TRUE);
+
+		recent = gedit_recent_get_model ();
+		gnome_recent_model_add (recent, canonical_uri_utf8);
+
+		g_free (raw_uri);
+		g_free (canonical_uri);
+		g_free (canonical_uri_utf8);
+
 		return TRUE;
 	}
 }
@@ -289,11 +338,13 @@ gedit_file_save (GeditMDIChild* child)
 gboolean
 gedit_file_save_as (GeditMDIChild *child)
 {
-	gchar* file;
+	gchar *file;
 	gboolean ret = FALSE;
 	GeditDocument *doc;
-	gchar* fname;
-
+	gchar *fname = NULL;
+	gchar *path = NULL;
+	gchar *uri = NULL;
+	
 	gedit_debug (DEBUG_FILE, "");
 
 	g_return_val_if_fail (child != NULL, FALSE);
@@ -301,30 +352,71 @@ gedit_file_save_as (GeditMDIChild *child)
 	doc = child->document;
 	g_return_val_if_fail (doc != NULL, FALSE);
 
-	fname = gedit_document_get_uri (doc);
+	uri = gedit_document_get_uri (doc);
+	g_return_val_if_fail (uri != NULL, FALSE);
 
+	if (gedit_document_is_untitled (doc))
+	{
+		path = (gedit_default_path != NULL) ? 
+			g_strdup (gedit_default_path) : NULL;
+		fname = uri;
+	}
+	else
+	{
+		fname = gnome_vfs_x_uri_get_basename (uri);
+
+		if (gedit_utils_uri_has_file_scheme (uri))
+			path = gnome_vfs_x_uri_get_dirname (uri);
+		else
+			path = (gedit_default_path != NULL) ? 
+				g_strdup (gedit_default_path) : NULL;
+	}
+				
+	g_return_val_if_fail (fname != NULL, FALSE);
+	
 	file = gedit_file_selector_save (
 			GTK_WINDOW (bonobo_mdi_get_active_window (BONOBO_MDI (gedit_mdi))),
 			FALSE,
 		        _("Save as ..."), 
 			NULL, 
-			NULL,
+			path,
 			fname);
+	
+	g_free (uri);
 	g_free (fname);
+	
+	if (path != NULL)
+		g_free (path);
 
-	if (file) 
+	if (file != NULL) 
 	{
-		gedit_utils_flash_va (_("Saving file '%s' ..."), file);
+		gchar *file_utf8;
+
+		file_utf8 = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
+
+		if (file_utf8 != NULL)
+			gedit_utils_flash_va (_("Saving file '%s' ..."), file_utf8);
 		
 		ret = gedit_file_save_as_real (file, child);
 		
 		if (ret)
-			gedit_utils_flash_va (_("File '%s' saved."), file);
+		{
+			if (file_utf8 != NULL)
+				gedit_utils_flash_va (_("File '%s' saved."), file_utf8);
+
+			if (gedit_default_path != NULL)
+				g_free (gedit_default_path);
+
+			gedit_default_path = gnome_vfs_x_uri_get_dirname (file);
+		}
 		else
 			gedit_utils_flash_va (_("The document has not been saved."));
 
 		gedit_debug (DEBUG_FILE, "File: %s", file);
 		g_free (file);
+
+		if (file_utf8 != NULL)
+			g_free (file_utf8);
 	}
 
 	return ret;
@@ -366,15 +458,25 @@ gedit_file_save_as_real (const gchar* file_name, GeditMDIChild *child)
 	}	
 	else
 	{
-		BonoboWindow *win;
-		GeditRecent *recent;
+		gchar *temp;
 
 		gedit_debug (DEBUG_FILE, "OK");
 
-		win = gedit_get_active_window ();
-		recent = gedit_mdi_get_recent_from_window (win);
-		gedit_recent_add (recent, uri);
+		/* uri is not valid utf8 */
+		
+		temp = g_filename_to_utf8 (uri, -1, NULL, NULL, NULL);
 
+		if (temp != NULL)
+		{
+			GnomeRecentModel *recent;
+
+			recent = gedit_recent_get_model ();
+			gnome_recent_model_add (recent, temp);
+
+			g_free (temp);
+		}
+
+		
 		g_free (uri);
 
 		return TRUE;
@@ -498,16 +600,20 @@ gedit_file_revert (GeditMDIChild *child)
 }
 
 static gchar *
-gedit_file_make_canonical_uri_from_shell_arg (const char *location)
+gedit_file_make_uri_from_shell_arg (const char *location)
 {
 	gchar* uri;
 	gchar* canonical_uri;
 	
-	gedit_debug (DEBUG_FILE, "");
+	gedit_debug (DEBUG_FILE, "LOCATION: %s", location);
 	
 	uri = gnome_vfs_x_make_uri_from_shell_arg (location);
 
-	canonical_uri = gnome_vfs_x_make_uri_canonical (uri);
+	gedit_debug (DEBUG_FILE, "URI: %s", uri);
+
+	canonical_uri = gnome_vfs_unescape_string_for_display (uri);
+
+	gedit_debug (DEBUG_FILE, "CANONICAL URI: %s", canonical_uri);
 
 	g_free (uri);
 
@@ -539,7 +645,7 @@ gedit_file_open_uri_list (GList* uri_list, gint line)
         /* create a file for each document in the parameter list */
 	for ( ; uri_list; uri_list = g_list_next (uri_list))
 	{
-		full_path = gedit_file_make_canonical_uri_from_shell_arg (uri_list->data);
+		full_path = gedit_file_make_uri_from_shell_arg (uri_list->data);
 		if (full_path != NULL) 
 		{
 			if (gedit_file_open_real (full_path, 
@@ -574,15 +680,28 @@ gedit_file_open_uri_list (GList* uri_list, gint line)
 }
 
 gboolean 
-gedit_file_open_recent (GeditRecent *recent, const gchar *uri, gpointer data)
+gedit_file_open_recent (GnomeRecentView *view, const gchar *uri, gpointer data)
 {
-	gboolean ret;
+	gboolean ret = FALSE;
 	GeditView* active_view;
+	gchar *temp;
 
 	gedit_debug (DEBUG_FILE, "Open : %s", uri);
 
-	ret = gedit_file_open_single_uri (uri);
+	temp = g_filename_from_utf8 (uri, -1, NULL, NULL, NULL);
 
+	if (temp != NULL)
+	{
+		ret = gedit_file_open_single_uri (temp);
+	
+		if (ret) {
+			GnomeRecentModel *model;
+
+			model = gedit_recent_get_model ();
+			gnome_recent_model_add (model, uri);
+		}
+	}
+		
 	active_view = gedit_get_active_view ();
 	if (active_view != NULL)
 		gtk_widget_grab_focus (GTK_WIDGET (active_view));
@@ -631,7 +750,18 @@ gedit_file_open_single_uri (const gchar* uri)
 		ret = gedit_file_open_real (full_path, 
 					    (active_child != NULL) ? GEDIT_MDI_CHILD (active_child): NULL);
 		if (ret)
-			gedit_utils_flash_va (_("Loaded file '%s'"), full_path);
+		{
+			gchar *uri_utf8;
+
+			uri_utf8 = g_filename_to_utf8 (full_path, -1, NULL, NULL, NULL);
+
+			if (uri_utf8 != NULL)
+			{
+				gedit_utils_flash_va (_("Loaded file '%s'"), uri_utf8);
+
+				g_free (uri_utf8);
+			}
+		}
 
 		g_free (full_path);
 	}
