@@ -27,6 +27,10 @@
  * See the ChangeLog files for a list of changes. 
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <libgnome/libgnome.h>
 #include <libgnomeprint/gnome-print.h>
 #include <libgnomeprint/gnome-print-master.h>
@@ -45,6 +49,9 @@
 #define A4_WIDTH (210.0 * 72 / 25.4)
 #define A4_HEIGHT (297.0 * 72 / 25.4)
 
+
+static GnomePrintConfig *gedit_print_config = NULL;
+
 typedef struct _GeditPrintJobInfo	GeditPrintJobInfo;
 
 struct _GeditPrintJobInfo {
@@ -79,7 +86,11 @@ struct _GeditPrintJobInfo {
 	gint			 printed_lines;
 };
 
-static GQuark gedit_print_error_quark ();
+
+/* FIXME: remove when libgnomeprint well define it -- Paolo */
+GnomeGlyphList *gnome_glyphlist_unref (GnomeGlyphList *gl);
+
+static GQuark gedit_print_error_quark (void);
 #define GEDIT_PRINT_ERROR gedit_print_error_quark ()	
 
 static GeditPrintJobInfo* gedit_print_job_info_new (GeditDocument* doc, GError **error);
@@ -125,6 +136,15 @@ gedit_print_job_info_new (GeditDocument* doc, GError **error)
 	g_return_val_if_fail (doc != NULL, NULL);
 	g_return_val_if_fail (gedit_settings != NULL, NULL);
 
+	if (gedit_print_config == NULL)
+	{
+		gedit_print_config = gnome_print_config_default ();
+		g_return_val_if_fail (gedit_print_config != NULL, NULL);
+
+		gnome_print_config_set (gedit_print_config, "Settings.Transport.Backend", "lpr");
+		gnome_print_config_set (gedit_print_config, "Printer", "GENERIC");
+	}
+
 	pji = g_new0 (GeditPrintJobInfo, 1);
 
 	pji->doc = doc;
@@ -173,6 +193,10 @@ gedit_print_job_info_new (GeditDocument* doc, GError **error)
 	pji->first_line_to_print = 1;
 	pji->printed_lines = 0;
 
+		
+	pji->config = gedit_print_config;
+	gnome_print_config_ref (pji->config);
+
 	return pji;
 }	
 
@@ -262,8 +286,11 @@ gedit_print_run_dialog (GeditPrintJobInfo *pji)
 	else
 		selection_flag = GNOME_PRINT_RANGE_SELECTION;
 		
-	dialog = gnome_print_dialog_new ((const char *) _("gedit - Print Document"),
-			          GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
+	dialog = g_object_new (GNOME_TYPE_PRINT_DIALOG, "print_config", pji->config, NULL);
+	
+	gnome_print_dialog_construct (GNOME_PRINT_DIALOG (dialog), 
+				      _("Print"),
+			              GNOME_PRINT_DIALOG_RANGE | GNOME_PRINT_DIALOG_COPIES);
 	
 	lines = gedit_document_get_line_count (pji->doc);
 	
@@ -294,9 +321,6 @@ gedit_print_run_dialog (GeditPrintJobInfo *pji)
 			return TRUE;
 	}
 	
-	g_return_val_if_fail (pji->config == NULL, TRUE);
-	pji->config = gnome_print_dialog_get_config (GNOME_PRINT_DIALOG (dialog));
-
 	pji->range_type = gnome_print_dialog_get_range (GNOME_PRINT_DIALOG (dialog));
 
 	gtk_widget_destroy (dialog);
@@ -499,8 +523,6 @@ gedit_print_real (GeditDocument* doc, gboolean preview, GError **error)
 		 * pji->config 
 		 */
 		cancel = gedit_print_run_dialog (pji);
-	else
-		pji->config = gnome_print_config_default ();
 
 	/* The canceled button on the dialog was clicked */
 	if (cancel) {
@@ -712,6 +734,8 @@ gedit_print_line_number (GeditPrintJobInfo *pji, gdouble y)
 		
 	gint line_num = pji->first_line_to_print + pji->printed_lines;
 
+	gedit_debug (DEBUG_PRINT, "");
+
 	if ((pji->printed_lines + 1) % gedit_settings->print_line_numbers != 0)
 	       return;	
 		
@@ -855,6 +879,8 @@ gedit_print_create_new_page (GeditPrintJobInfo *pji)
 {
 	gchar *pn_str;
 
+	gedit_debug (DEBUG_PRINT, "");
+
 	++pji->page_num;
 	pn_str = g_strdup_printf ("%d", pji->page_num);
 	gnome_print_beginpage (pji->print_ctx, pn_str);
